@@ -22,7 +22,11 @@
 
 #include "gda-data-model-import.h"
 #include <unistd.h>
-#include <sys/mman.h>
+#ifndef G_OS_WIN32
+  #include <sys/mman.h>
+#else
+  #include <windows.h>
+#endif
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -146,9 +150,9 @@ static void                 gda_data_model_import_data_model_init (GdaDataModelC
 static gint                 gda_data_model_import_get_n_rows      (GdaDataModel *model);
 static gint                 gda_data_model_import_get_n_columns   (GdaDataModel *model);
 static GdaColumn           *gda_data_model_import_describe_column (GdaDataModel *model, gint col);
-static guint                gda_data_model_import_get_access_flags(GdaDataModel *model);
+static GdaDataModelAccessFlags gda_data_model_import_get_access_flags(GdaDataModel *model);
 static const GValue      *gda_data_model_import_get_value_at    (GdaDataModel *model, gint col, gint row);
-static guint                gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row);
+static GdaValueAttribute    gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row);
 static GdaDataModelIter    *gda_data_model_impor_create_iter      (GdaDataModel *model);
 static gboolean             gda_data_model_import_iter_next       (GdaDataModel *model, GdaDataModelIter *iter); 
 static gboolean             gda_data_model_import_iter_prev       (GdaDataModel *model, GdaDataModelIter *iter);
@@ -315,15 +319,20 @@ gda_data_model_import_dispose (GObject *object)
 
 		/* data access mem free */
 		if (model->priv->is_mapped) {
+			if (model->priv->src.mapped.start) {
+#ifndef G_OS_WIN32
+				munmap (model->priv->src.mapped.start, model->priv->src.mapped.length);
+#else
+				UnmapViewOfFile (model->priv->src.mapped.start);
+#endif
+				model->priv->src.mapped.start = NULL;
+			}
+
 			g_free (model->priv->src.mapped.filename);
 			if (model->priv->src.mapped.fd >= 0) {
 				close (model->priv->src.mapped.fd);
 				model->priv->src.mapped.fd = -1;
 			}
-			if (model->priv->src.mapped.start) {
-				munmap (model->priv->src.mapped.start, model->priv->src.mapped.length);
-				model->priv->src.mapped.start = NULL;
-			}
 		}
 		else {
 			g_free (model->priv->src.string);
@@ -490,6 +499,7 @@ gda_data_model_import_set_property (GObject *object,
 					return;
 				}
 				model->priv->src.mapped.length = _stat.st_size;
+#ifndef G_OS_WIN32
 				model->priv->src.mapped.start = mmap (NULL, model->priv->src.mapped.length,
 								      PROT_READ, MAP_PRIVATE, 
 								      model->priv->src.mapped.fd, 0);
@@ -499,6 +509,22 @@ gda_data_model_import_set_property (GObject *object,
 					model->priv->src.mapped.start = NULL;
 					return;
 				}
+#else
+				HANDLE view = CreateFileMapping((HANDLE)model->priv->src.mapped.fd, 
+								NULL, PAGE_READONLY|SEC_COMMIT, 0,0 , NULL);
+				if (!view) {
+					/* error */
+					add_error (model, sys_errlist [errno]);
+					return;
+				}
+				model->priv->src.mapped.start = MapViewOfFile(view, FILE_MAP_READ, 0, 0, 
+									      model->priv->src.mapped.length);
+				if (!model->priv->src.mapped.start) {
+					/* error */
+					add_error (model, sys_errlist [errno]);
+					return;
+				}
+#endif
 				model->priv->data_start = model->priv->src.mapped.start;
 				model->priv->data_length = model->priv->src.mapped.length;
 			}
@@ -627,11 +653,25 @@ gda_data_model_import_get_property (GObject *object,
 /**
  * gda_data_model_import_new_file
  * @filename: the file to import data from
- * @random_access: TRUE if a random acces will be required
- * @options: list of options for the export
+ * @random_access: TRUE if random access will be required
+ * @options: list of importing options
  *
  * Creates a new #GdaDataModel object which contains the data stored within the @filename file.
  *
+ * The options are the following ones:
+ * <itemizedlist>
+ *   <listitem><para>For the CSV format:
+ *      <itemizedlist>
+ *         <listitem><para>ENCODING (string): specifies the encoding of the data in the file</para></listitem>
+ *         <listitem><para>SEPARATOR (string): specifies the CSV separator</para></listitem>
+ *         <listitem><para>ESCAPE_CHAR (string): specifies the character used to "escape"</para></listitem>
+ *         <listitem><para>TITLE_AS_FIRST_LINE (boolean): consider that the first line of the file contains columns' titles</para></listitem>
+ *         <listitem><para>DBMS_TYPE_&lt;column number&gt; (string): specifies the type of value expected in column &lt;column number&gt;</para></listitem>
+ *      </itemizedlist>
+ *   </para></listitem>
+ *   <listitem><para>Other formats: no option</para></listitem>
+ * </itemizedlist>
+ *
  * Returns: a pointer to the newly created #GdaDataModel.
  */
 GdaDataModel *
@@ -653,6 +693,8 @@ gda_data_model_import_new_file   (const gchar *filename, gboolean random_access,
 /**
  * gda_data_model_import_new_mem
  * @data: a string containng the data to import
+ * @random_access: TRUE if random access will be required
+ * @options: list of importing options, see gda_data_model_import_new_file() for more information
  *
  * Creates a new #GdaDataModel object which contains the data stored in the @data string. 
  *
@@ -1301,7 +1343,11 @@ init_xml_import (GdaDataModelImport *model)
 		
 		prop = (gchar*)xmlGetProp (node, (xmlChar*)"id");
 		if (prop) {
-			gda_object_set_id (GDA_OBJECT (model), prop);
+			/* use prop+2 only if prop is like "DA%d", otherwise don't set object's ID */
+			if ((prop[0] == 'D') && (prop[1] == 'A'))
+				gda_object_set_id (GDA_OBJECT (model), prop+2);
+			else
+				gda_object_set_id (GDA_OBJECT (model), NULL);
 			xmlFree (prop);
 		}
 		prop = (gchar*)xmlGetProp (node, (xmlChar*)"name");
@@ -1523,56 +1569,97 @@ xml_fetch_next_row (GdaDataModelImport *model)
 		if (!columns) 
 			add_error (model, _("Row has too many values (which are ignored)"));
 		else {
-			ret = xmlTextReaderRead (reader);
-			if (ret > 0) {
-				GValue *value;
-				GdaColumn *column;
+			gboolean value_is_null = FALSE;
+			xmlChar *isnull;
+			GdaColumn *column;
 
-				if (this_lang)
-					column = last_column;
-				else {
-					column = (GdaColumn *) columns->data;
-					last_column = column;
-					columns = g_slist_next (columns);
-				}
-				if (xmlTextReaderNodeType (reader) == XML_TEXT_NODE) {
+			isnull = xmlTextReaderGetAttribute (reader, (xmlChar*)"isnull");
+			if (isnull) {
+				if ((*isnull == 't') || (*isnull == 'T'))
+					value_is_null = TRUE;
+				xmlFree (isnull);
+			}
+			
+			if (this_lang)
+				column = last_column;
+			else {
+				column = (GdaColumn *) columns->data;
+				last_column = column;
+				columns = g_slist_next (columns);
+			}
+
+			if (value_is_null) {
+				values = g_slist_prepend (values, gda_value_new_null ());
+				ret = -1;
+			}
+			else {
+				ret = xmlTextReaderRead (reader);
+				if (ret > 0) {
+					GValue *value;
 					GType gtype;
-					const xmlChar *xmlstr;
-
+					
+					ret = -1;
+					
 					gtype = gda_column_get_g_type (column);
-					xmlstr = xmlTextReaderConstValue (reader);
-					/* g_print ("Convert #%s# to %s\n", (gchar *) xmlstr, gda_g_type_to_string (gtype)); */
-					value = gda_value_new_from_string ((gchar *) xmlstr, gtype);
-					if (!value) {
-						gchar *str;
+					/*g_print ("TYPE: %s\n", xmlTextReaderConstName (reader));*/
+					if (xmlTextReaderNodeType (reader) == XML_TEXT_NODE) {
+						const xmlChar *xmlstr;
 						
-						str = g_strdup_printf (_("Could not convert '%s' to a value of type %s"), 
-								       (gchar *) xmlstr, gda_g_type_to_string (gtype));
-						add_error (model, str);
-						g_free (str);
-						value = gda_value_new_null ();
+						xmlstr = xmlTextReaderConstValue (reader);
+						value = gda_value_new_from_string ((gchar *) xmlstr, gtype);
+						/*g_print ("Convert #%s# (type:%s) => %s (type:%s)\n", (gchar *) xmlstr, 
+							 gda_g_type_to_string (gtype), 
+							 gda_value_stringify (value), 
+							 value ? gda_g_type_to_string (G_VALUE_TYPE (value)) : "no type");*/
+						if (!value) {
+							gchar *str;
+							
+							str = g_strdup_printf (_("Could not convert '%s' to a value of type %s"), 
+									       (gchar *) xmlstr, gda_g_type_to_string (gtype));
+							add_error (model, str);
+							g_free (str);
+							value = gda_value_new_null ();
+						}
 					}
+					else {
+						if (xmlTextReaderNodeType (reader) == XML_ELEMENT_NODE)
+							ret = 1; /* don't read another node in the next loop, 
+								    use the current one */
+						if (value_is_null)
+							value = gda_value_new_null ();
+						else
+							value = gda_value_new_from_string ("", gtype);
+					}
+					
+					if (this_lang) {
+						/* replace the last value (which did not have any "lang" attribute */
+						gda_value_free ((GValue *) values->data);
+						values->data = value;
+						xmlFree (this_lang);
+					}
+					else 
+						values = g_slist_prepend (values, value);
 				}
-				else 
-					/* no contents => this is a NULL value */
-					value = gda_value_new_null ();
-
-				if (this_lang) {
-					/* replace the last value (which did not have any "lang" attribute */
-					gda_value_free ((GValue *) values->data);
-					values->data = value;
-					xmlFree (this_lang);
-				}
-				else 
-					values = g_slist_prepend (values, value);
 			}
 		}
-		ret = xml_fetch_next_xml_node (reader);
+		if (ret < 0) /* read another node ? */
+			ret = xml_fetch_next_xml_node (reader);
 		name = (ret > 0) ? xmlTextReaderConstName (reader) : NULL;
 	}
 
 	if (values)
 		model->priv->cursor_values = g_slist_reverse (values);
+#ifdef GDA_DEBUG_NO
+	GSList *l;
+	gint c;
+	
+	g_print ("======== GdaDataModelImport => next XML row ========\n");
+	for (c = 0, l = model->priv->cursor_values; l; l = l->next, c++) {
+		GValue *val = (GValue*)(l->data);
+		g_print ("#%d %s (type:%s)\n", c, gda_value_stringify (val),
+			 val ? g_type_name (G_VALUE_TYPE (val)) : "no type");
+	}
+#endif
 
 	if (ret <= 0) {
 		/* destroy the reader, nothing to read anymore */
@@ -1689,8 +1776,11 @@ init_node_import (GdaDataModelImport *model)
 	model->priv->random_access_model = ramodel;
 	str = (gchar*)xmlGetProp (node, (xmlChar*)"id");
 	if (str) {
-		/* FIXME: use str+2 only if str is like "DA%d", otherwise simply use str */
-		gda_object_set_id (GDA_OBJECT (model), str+2);
+		/* use str+2 only if str is like "DA%d", otherwise don't set object's ID */
+		if ((str[0] == 'D') && (str[1] == 'A'))
+			gda_object_set_id (GDA_OBJECT (model), str+2);
+		else
+			gda_object_set_id (GDA_OBJECT (model), NULL);
 		xmlFree (str);
 	}
 	str = (gchar*)xmlGetProp (node, (xmlChar*)"name");
@@ -1837,11 +1927,11 @@ gda_data_model_import_describe_column (GdaDataModel *model, gint col)
 		return NULL;
 }
 
-static guint
+static GdaDataModelAccessFlags
 gda_data_model_import_get_access_flags (GdaDataModel *model)
 {
 	GdaDataModelImport *imodel;
-	guint flags = GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD;
+	GdaDataModelAccessFlags flags = GDA_DATA_MODEL_ACCESS_CURSOR_FORWARD;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_IMPORT (model), 0);
 	imodel = GDA_DATA_MODEL_IMPORT (model);
@@ -1872,10 +1962,10 @@ gda_data_model_import_get_value_at (GdaDataModel *model, gint col, gint row)
 		return NULL;
 }
 
-static guint
+static GdaValueAttribute
 gda_data_model_import_get_attributes_at (GdaDataModel *model, gint col, gint row)
 {
-	guint flags = 0;
+	GdaValueAttribute flags = 0;
 	GdaDataModelImport *imodel;
 
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_IMPORT (model), 0);
