@@ -152,7 +152,7 @@ gda_data_model_query_get_type (void)
 {
 	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
+	if (!type) {
 		static const GTypeInfo info = {
 			sizeof (GdaDataModelQueryClass),
 			(GBaseInitFunc) NULL,
@@ -382,8 +382,7 @@ static void
 check_param_type (GdaParameter *param, GType type)
 {
 	if ((type != 0) && (type !=  gda_parameter_get_g_type (param))) {
-		g_warning (_("Type of parameter '%s' is '%s' when it should be '%s', "
-			     "GdaDataModelQuery object will not work correctly"),
+		g_warning (_("Type of parameter '%s' is '%s' when it should be '%s', GdaDataModelQuery object will now work correctly"),
 			   gda_object_get_name (GDA_OBJECT (param)), 
 			   g_type_name (gda_parameter_get_g_type (param)),
 			   g_type_name (type));
@@ -593,7 +592,7 @@ param_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GdaDataModel
 			g_object_unref (model->priv->data);
 			model->priv->data = NULL;
 		}
-		gda_data_model_reset ((GdaDataModel *) model);
+		gda_data_model_signal_emit_changed ((GdaDataModel *) model);
 	}
 }
 
@@ -684,7 +683,7 @@ gda_data_model_query_refresh (GdaDataModelQuery *model, GError **error)
 		return FALSE;
 	}
 
-	gda_data_model_reset ((GdaDataModel *) model);
+	gda_data_model_signal_emit_changed ((GdaDataModel *) model);
 	return model->priv->data ? TRUE : FALSE;
 }
 
@@ -885,8 +884,7 @@ create_columns (GdaDataModelQuery *model)
 			}
 		}
 		else {
-			/* Can't compute the list of columns because the SELECT query is not active 
-			 * and the model does not contain any data;
+			/* Can't compute the list of columns because the SELECT query is not active and the model does not contain any data;
 			 * next time we emit a "reset" signal */
 			model->priv->emit_reset = TRUE;
 		}
@@ -1666,7 +1664,7 @@ auto_compute_assert_modify_target (GdaDataModelQuery *model, const gchar *target
 		if (table_targets->next) {
 			g_set_error (error, GDA_DATA_MODEL_QUERY_ERROR,
 				     GDA_DATA_MODEL_QUERY_COMPUTE_MODIF_QUERIES_ERROR,
-				     _("More than one target table can be modified in SELECT query, specify which one"));
+				     _("Ambiguous target table to modify in SELECT query"));
 			g_slist_free (table_targets);
 			g_slist_free (targets);
 			return FALSE;
@@ -1747,10 +1745,6 @@ auto_compute_make_cond_query_fields (GdaDataModelQuery *model, GdaQueryTarget *m
 			/* use ALL the present fields to make a WHERE condition */
 			cond_query_fields = gda_query_get_fields_by_target (model->priv->queries[SEL_QUERY],
 									    modify_target, TRUE);
-			if (error_is_set && error) {
-				g_error_free (*error);
-				*error = NULL;
-			}
 		}
 		else {
 			if (!error_is_set) {
