@@ -142,7 +142,7 @@ gda_dict_database_get_type (void)
 {
 	static GType type = 0;
 
-	if (G_UNLIKELY (type == 0)) {
+	if (!type) {
 		static const GTypeInfo info = {
 			sizeof (GdaDictDatabaseClass),
 			(GBaseInitFunc) NULL,
@@ -299,7 +299,7 @@ gda_dict_database_class_init (GdaDictDatabaseClass * class)
 	object_class->dispose = gda_dict_database_dispose;
 	object_class->finalize = gda_dict_database_finalize;
 
-#if 0 /* This object does not have any properties.*/
+        #if 0 /* This object does not have any properties.
 	/* Properties */
 	object_class->set_property = gda_dict_database_set_property;
 	object_class->get_property = gda_dict_database_get_property;
@@ -1070,11 +1070,11 @@ database_tables_update_list (GdaDictDatabase *db, const gchar *table_name, GErro
 
 	/* tables */
 	rs = gda_connection_get_schema (cnc, GDA_CONNECTION_SCHEMA_TABLES, options, NULL);
+	if (options)
+		g_object_unref (options);
 	if (!rs) {
 		g_set_error (error, GDA_DICT_DATABASE_ERROR, GDA_DICT_DATABASE_TABLES_ERROR,
 			     _("Can't get list of tables"));
-		if (options) 
-			g_object_unref (options);
 		return FALSE;
 	}
 	updated_tables = database_tables_n_views_update_treat_schema_result (db, rs, FALSE, error);
@@ -1083,11 +1083,8 @@ database_tables_update_list (GdaDictDatabase *db, const gchar *table_name, GErro
 
 	/* views */
 	rs = gda_connection_get_schema (cnc, GDA_CONNECTION_SCHEMA_VIEWS, options, NULL);
-	if (options) {
+	if (options)
 		g_object_unref (options);
-		options = NULL;
-	}
-
 	if (!rs) {
 		g_set_error (error, GDA_DICT_DATABASE_ERROR, GDA_DICT_DATABASE_TABLES_ERROR,
 			     _("Can't get list of views"));
