@@ -2,20 +2,20 @@
  *
  * Copyright (C) 2006 Vivien Malerba
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
+ * This Library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This Library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this Library; see the file COPYING.LIB.  If not,
+ * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #include <string.h>
@@ -82,11 +82,11 @@ types_load_xml_tree (GdaDict *dict, xmlNodePtr types, GError **error)
 	gboolean allok = TRUE;
 	
 	while (qnode && allok) {
-		if (!strcmp (qnode->name, XML_ELEM_TAG)) {
+		if (!strcmp ((gchar*)qnode->name, XML_ELEM_TAG)) {
 			gchar *prop;
 			gboolean load = TRUE;
 
-			prop = xmlGetProp (qnode, "custom");
+			prop = (gchar*)xmlGetProp(qnode, (xmlChar*)"custom");
 			if (prop) {
 				if (*prop == 't')
 					load = FALSE;
@@ -140,7 +140,7 @@ types_save_xml_tree (GdaDict *dict, xmlNodePtr group, GError **error)
 		qnode = gda_xml_storage_save_to_xml (GDA_XML_STORAGE (list->data), error);
 		if (qnode) {
 			xmlAddChild (group, qnode);
-			xmlSetProp (qnode, "custom", "t");
+			xmlSetProp(qnode, (xmlChar*)"custom", (xmlChar*)"t");
 		}
 		else 
 			/* error handling */
